# snap-ml-spark-examples

This repository contains examples of using the **snap-ml-spark** module.

## Install and Configure Spark

The following instructions assume that you have access to a cluster with:
1. A home directory that is shared across the machines
2. SSH is configured with password-less authentication (e.g. RSA)

First, let's download Spark:
```bash
cd ~
wget https://archive.apache.org/dist/spark/spark-2.3.0/spark-2.3.0-bin-hadoop2.7.tgz
tar xzf spark-2.3.0-bin-hadoop2.7.tgz 
cd spark-2.3.0-bin-hadoop2.7
```
Let's assume we want to start a cluster with the following topology
```
master: node0011
workers: node005, node006, node007, node008
```
Edit the `conf/slaves` and list the workers:
```
node005
node006
node007
node008
```

Make sure you assign all the cores to the worker running in each node and this will start one executor per node with all the cores assigned to it.

For using snap_ml_spark with GPUs it is recommended to set `SPARK_WORKER_INSTANCES` to the number of GPUs on each node and to set `SPARK_WORKER_CORES` to be 1. It is important to change the `SPARK_LOCAL_DIRS` environment variable to point to a *local* directory on each worker that your user has write access to. If this directory is shared between workers then the GPU selection will not work. 

To finish configuration of Spark, edit the `conf/spark-defaults.conf` file:
```
spark.master              spark://node011.ibm.com:7077
spark.executor.memory     16g
spark.solver.count        4
spark.locality.wait.node  100
```
Now let's start the Spark cluster. 
```bash
ssh node011
cd spark-2.1.1-bin-hadoop2.7
./sbin/start-all.sh
```

Please note that spark has to be configured also in each individual python example (e.g. application name,path to test and train data, etc...).

For example:
```python
## SPARK

from pyspark import SparkConf, SparkContext
from pyspark.sql.session import SparkSession

conf = SparkConf()\
.set("spark.solver.count",args.num_partitions)\
.setAppName("Spark-criteoKaggle2014-SparkComparison")\
.set("spark.executor.extraJavaOptions","-XX:MaxDirectMemorySize=8192m")\
.set("spark.driver.maxResultSize",8192)
# Create spark config
sc = SparkContext(conf=conf)
# Configure Spark
spark = SparkSession(sc)


# disable spark INFO logging - comment out to enable
log4j = sc._jvm.org.apache.log4j
log4j.LogManager.getRootLogger().setLevel(log4j.Level.ERROR)
```
One can set the master to `local` to run the examples in the spark standalone mode without needing to set the previous spark configuration.

Please note that if you wish to run Spark on a real distributed environment, then you must set the absolute data paths of the train/test sets and also have access to the data from all the connected machines that will be used (by either replicating the data on each of the available machines or by using a distributed file system).

##Install dependent packages and configure paths


  SPARK_HOME environment variable must be set to Apache/HDP Spark installation directory as:
  export SPARK_HOME=~/spark-2.3.1-bin-hadoop2.7

  source $CONDA_PREFIX/bin/snap-ml-spark-activate

## Running the examples
To run a python example using the snap_ml_spark module, use the following command after completing your spark configuration

```bash
spark-submit --master spark://node1.ibm.com:7077 --jars <path_to_snap_ml_spark_jar> <example>.py
```

In case of any memory issue, please increase the spark driver memory for `local` mode and `executor memory` for `cluster` mode:

```bash
spark-submit --master spark://node1.ibm.com:7077 --jars <path_to_snap_ml_spark_jar> --driver-memory 2g <example>.py
```

Use the following command to submit a job to HDP Yarn cluster in client mode. 

```bash
spark-submit --master yarn --deploy-mode client --jars <path_to_snap_ml_spark_jar> <example>.py
```
Use the below command to submit a job to HDP Yarn cluster in cluster mode.

```bash
spark-submit --master yarn --deploy-mode cluster --jars <path_to_snap_ml_spark_jar> <example>.py
```

The example scripts which show comparison of execution times with SnapML and SparkML provide a command line option --skip_sparkml to skip execution of the part of example code that uses Spark ML.

Please see the invidual sub-directories for step-by-step instructions for running each of the examples.

&copy; Copyright IBM Corporation 2018, 2019
